#include "Model.hpp"

Rock::Rock(Vector3f _pos, float _size, float pt, Graphics::TextureRef _tex, Graphics::TextureRef _tex2) {
  pos = _pos;
  size = _size;
  tex = _tex;
  tex2 = _tex2;
  exorcised = false;
  purge_time = pt;
}

Rock::~Rock() {

}

void Rock::exorcise(float purge_delta) {
  if(purge_time <= 0.0f) {
    exorcised = true;
  }
  purge_time -= purge_delta;

}

bool Rock::is_exorcised() {
  return exorcised;
}

void Rock::draw() {
  glPushMatrix();
  glTranslatef(pos.x, pos.y, pos.z);
  if(exorcised) {
    glBindTexture(GL_TEXTURE_2D, tex2->obj);
  }
  else {
    glBindTexture(GL_TEXTURE_2D, tex->obj);
  }

  glBegin(GL_QUADS);

  float s = size / 2.0f;
  float ns = -s;

  //face 1
  glNormal3f(0.0f, -1.0f, 0.0f);
  glTexCoord2f(0.0f, 0.0f);
  glVertex3f(ns, ns, ns);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(ns, ns, s);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f(s, ns, s);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f(s, ns, ns);
  
  //face 2
  glNormal3f(0.0f, 1.0f, 0.0f);
  glTexCoord2f(0.0f, 0.0f);
  glVertex3f(s, s, ns);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(s, s, s);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f(ns, s, s);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f(ns, s, ns);

  //face 3

  glNormal3f(-1.0f, 0.0f, 0.0f);
  glTexCoord2f(0.0f, 0.0f);
  glVertex3f(ns, ns, ns);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(ns, ns, s);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f(ns, s, s);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f(ns, s, ns);
  
  //face 4

  glNormal3f(1.0f, 0.0f, 0.0f);
  glTexCoord2f(0.0f, 0.0f);
  glVertex3f(s, ns, ns);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(s, ns, s);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f(s, s, s);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f(s, s, ns);
  
  //face 5

  glNormal3f(0.0f, 0.0f, -1.0f);
  glTexCoord2f(0.0f, 0.0f);
  glVertex3f(ns, ns, ns);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(s, ns, ns);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f(s, s, ns);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f(ns, s, ns);
  
  //face 6

  glNormal3f(0.0f, 0.0f, 1.0f);
  glTexCoord2f(0.0f, 0.0f);
  glVertex3f(ns, ns, s);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(s, ns, s);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f(s, s, s);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f(ns, s, s);
  
  glEnd();
  
  glPopMatrix();
}
