#include "Play.hpp"

Play::Play() {
   
  uptime = 0.0f;      
  player_pos = make_vector(-HORIZON, 0.0f, HORIZON, 1.0f);
  player_lookat = make_vector(0.0f, 0.0f, -2*HORIZON, 1.0f);
  player_rotation = make_vector(0.0f, 0.0f);
  mouse_pos = make_vector(0.0f, 0.0f);
  mouse_delta = make_vector(0.0f, 0.0f);
  score = 0;
  track = 0;

  //Other GL init code is in main.cpp

  GLfloat diffuse[] = {1.0, 1.0, 1.0, 0.7};
  GLfloat light_pos[] = {0.0, 0.0, 0.0, 1.0};
  GLfloat light_dir[] = {0.0f, 0.0f, -1.0f};
  glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, light_dir);
  glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 15.0f);

  //turn on the light
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  
  
  glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_LINE_SMOOTH);
  glEnable(GL_NORMALIZE);
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_DEPTH_TEST);

  //http://dr-photoshop.com/zfile/tutorials/planet/RockTextureA.jpg
  rockTexture = Graphics::get_texture("textures/rockTexture.png");
 
  //http://www.bittbox.com/~caleb/free_high_res_texture_70.jpg
  explodedTexture = Graphics::get_texture("textures/explodedTexture.png");

  for(unsigned int i = 0; i < 50; i++) {
    //the bigger the rock, the harder the exorcism
    float size = rand_in_window(false, 3.0f, 1.0f);

    Rock r = Rock(make_vector(
             rand_in_window(true, 10.0f, 0.0f), 
             rand_in_window(true, 10.0f, 0.0f),
             rand_in_window(true, 10.0f, 0.0f)),          
             size,
             size,
             rockTexture, explodedTexture);
    rocks.push_back(r);
  }

}

Play::~Play() {
}

void Play::draw() {
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
  
  //glLoadIdentity();
  glPushMatrix();
  //the Graphics::Font code disables this!
  glEnable(GL_TEXTURE_2D);
  
  glRotatef(player_rotation.x, 1.0, 0.0, 0.0);
  glRotatef(player_rotation.y, 0.0, 1.0, 0.0);
  gluLookAt(player_pos.x, player_pos.y, player_pos.z,
            player_lookat.x, player_lookat.y, player_lookat.z,
            0.0f, 1.0f, 0.0f);
  //glTranslatef(-player_pos.x, -player_pos.y, -player_pos.z);
  
  for(unsigned int i = 0; i < rocks.size(); i++) {
    rocks[i].draw();
  }  

  float floats[16];
  glGetFloatv(GL_MODELVIEW_MATRIX, floats);
  for(unsigned int i = 0; i < 16; ++i) {
    m.c[i] = floats[i];
  }
  glPopMatrix();
}

void Play::update(float elapsed_time) {
	//Update game state here.
  uptime += elapsed_time;
  
  if(track > 3) {
    return;
  }

  Vector4f transformed_lookat = (m * player_lookat);
  Vector3f normalized_lookat = normalize(transformed_lookat.xyz);
  Vector4f transformed_player_pos = player_pos;
    
  vector<Rock>::iterator r;
  for(r = rocks.begin(); r != rocks.end(); ++r) {
    
    if(r->exorcised) {
      continue;
    }
    
    Vector4f rock_pos = make_vector(r->pos.x, r->pos.y, r->pos.z, 1.0f);
    Vector4f transformed_rock_pos = rock_pos;
    //http://www.cs.cmu.edu/~fp/courses/graphics/cmu-only/ray2.pdf
   
    /* Algorithm for ray-sphere intersection taken from Introduction to Ray Tracing, Chapter 2 */
    /* Step 1: Calculate A, B, and C in the quadratic equation for the intersection */
    /* A = 1.0, unnecessary to calculate */
    float b = 2.0 * (normalized_lookat.x * (transformed_player_pos.x - transformed_rock_pos.x) + normalized_lookat.y * (transformed_player_pos.y - transformed_rock_pos.y) + normalized_lookat.z * (transformed_player_pos.z - transformed_rock_pos.z));
    float c = (pow((transformed_player_pos.x - transformed_rock_pos.x), 2.0) + pow((transformed_player_pos.y - transformed_rock_pos.y), 2.0) + pow((transformed_player_pos.z - transformed_rock_pos.z), 2.0)) - pow(r->size / 2.0f, 2.0);

    /* Step 2: Calculate the discriminant */
    float discriminant = (pow(b, 2.0)) - 4.0 * c;
    
    /* If the discriminant is negative, the line misses the sphere */
    if(discriminant < 0.0) {
      continue;
    }

    r->exorcise(elapsed_time);  
  }

  switch(track) {
    case 0:
      player_pos.z -= 0.02f;
      if(player_pos.z <= -HORIZON) {
        track++;
      }
      break;
    case 1:
      player_pos.x += 0.02f;
      if(player_pos.x >= HORIZON) {
        track++;
      }
      break;
    case 2:
      player_pos.z += 0.02f;
      if(player_pos.z >= HORIZON) {
        track++;
      }
      break;
    case 3:
      player_pos.x -= 0.02f;
      if(player_pos.x <= -HORIZON) {
        track++;
        for(unsigned int i = 0; i < rocks.size(); ++i) {
          if(rocks[i].is_exorcised()) {
            score++;
          }
        }
        std::cout << "Game over, you exorcised " << score << "/" << rocks.size() << " ghosts." << std::endl;
      }
      break;
    default:
      break;
  }

}

void Play::handle_event(SDL_Event const &event) {
	//Handle events here:
	if (event.type == SDL_KEYDOWN) {
		if (event.key.keysym.sym == SDLK_ESCAPE) {
			//escape quits.
			quit_screen(this);
		}
	}
  if (event.type == SDL_MOUSEMOTION) {
    float temp_x = event.motion.x / float(Graphics::screen.x) * Graphics::aspectf() * 2.0f - Graphics::aspectf();
    float temp_y = event.motion.y / float(Graphics::screen.y) *-2.0f + 1.0f;
    mouse_delta = make_vector(temp_x, temp_y) - mouse_pos;
    mouse_pos = make_vector(temp_x, temp_y);
    player_rotation.x -= mouse_delta.y * 60.0f;
    player_rotation.y += mouse_delta.x * 240.0f;
    //std::cout << "player_rotation:" << player_rotation.x << "      " << player_rotation.y << std::endl; 
  }

}

float Play::rand_in_window(bool allowNegative, float range, float offset) {
  float accum = rand() / float(RAND_MAX);
  accum *= range;
  accum += offset;
  if(allowNegative && rand() % 2) {
    accum = 0.0f - accum;
  }

  return accum;
}


void printvec(Vector3f v) {
  std::cout << v.x << ", " << v.y << ", " << v.z << std::endl;
}
void printvec(Vector4f v) {
  std::cout << v.x << ", " << v.y << ", " << v.z << ", " << v.w <<  std::endl;
}
